@@ -88,17 +88,17 @@ type matcherFunc func(chunks []string) (int, float64)
 //
 // The space argument is an empty slice with spare capacity that may be used
 // to allocate the result.
-type symbolizer func(space []string, name string, pkg Metadata, m matcherFunc) ([]string, float64)
+type symbolizer func(space []string, name string, pkg *Metadata, m matcherFunc) ([]string, float64)
 
-func fullyQualifiedSymbolMatch(space []string, name string, pkg Metadata, matcher matcherFunc) ([]string, float64) {
+func fullyQualifiedSymbolMatch(space []string, name string, pkg *Metadata, matcher matcherFunc) ([]string, float64) {
 	if _, score := dynamicSymbolMatch(space, name, pkg, matcher); score > 0 {
-		return append(space, pkg.PackagePath(), ".", name), score
+		return append(space, string(pkg.PkgPath), ".", name), score
 	}
 	return nil, 0
 }
 
-func dynamicSymbolMatch(space []string, name string, pkg Metadata, matcher matcherFunc) ([]string, float64) {
-	if IsCommandLineArguments(pkg.PackageID()) {
+func dynamicSymbolMatch(space []string, name string, pkg *Metadata, matcher matcherFunc) ([]string, float64) {
+	if IsCommandLineArguments(pkg.ID) {
 		// command-line-arguments packages have a non-sensical package path, so
 		// just use their package name.
 		return packageSymbolMatch(space, name, pkg, matcher)
@@ -106,14 +106,14 @@ func dynamicSymbolMatch(space []string, name string, pkg Metadata, matcher match
 
 	var score float64
 
-	endsInPkgName := strings.HasSuffix(pkg.PackagePath(), pkg.PackageName())
+	endsInPkgName := strings.HasSuffix(string(pkg.PkgPath), string(pkg.Name))
 
 	// If the package path does not end in the package name, we need to check the
 	// package-qualified symbol as an extra pass first.
 	if !endsInPkgName {
-		pkgQualified := append(space, pkg.PackageName(), ".", name)
+		pkgQualified := append(space, string(pkg.Name), ".", name)
 		idx, score := matcher(pkgQualified)
-		nameStart := len(pkg.PackageName()) + 1
+		nameStart := len(pkg.Name) + 1
 		if score > 0 {
 			// If our match is contained entirely within the unqualified portion,
 			// just return that.
@@ -126,11 +126,11 @@ func dynamicSymbolMatch(space []string, name string, pkg Metadata, matcher match
 	}
 
 	// Now try matching the fully qualified symbol.
-	fullyQualified := append(space, pkg.PackagePath(), ".", name)
+	fullyQualified := append(space, string(pkg.PkgPath), ".", name)
 	idx, score := matcher(fullyQualified)
 
 	// As above, check if we matched just the unqualified symbol name.
-	nameStart := len(pkg.PackagePath()) + 1
+	nameStart := len(pkg.PkgPath) + 1
 	if idx >= nameStart {
 		return append(space, name), score
 	}
@@ -139,9 +139,9 @@ func dynamicSymbolMatch(space []string, name string, pkg Metadata, matcher match
 	// initial pass above, so check if we matched just the package-qualified
 	// name.
 	if endsInPkgName && idx >= 0 {
-		pkgStart := len(pkg.PackagePath()) - len(pkg.PackageName())
+		pkgStart := len(pkg.PkgPath) - len(pkg.Name)
 		if idx >= pkgStart {
-			return append(space, pkg.PackageName(), ".", name), score
+			return append(space, string(pkg.Name), ".", name), score
 		}
 	}
 
@@ -150,8 +150,8 @@ func dynamicSymbolMatch(space []string, name string, pkg Metadata, matcher match
 	return fullyQualified, score * 0.6
 }
 
-func packageSymbolMatch(space []string, name string, pkg Metadata, matcher matcherFunc) ([]string, float64) {
-	qualified := append(space, pkg.PackageName(), ".", name)
+func packageSymbolMatch(space []string, name string, pkg *Metadata, matcher matcherFunc) ([]string, float64) {
+	qualified := append(space, string(pkg.Name), ".", name)
 	if _, s := matcher(qualified); s > 0 {
 		return qualified, s
 	}
@@ -298,14 +298,16 @@ func (c comboMatcher) match(chunks []string) (int, float64) {
 //   - A symbolizer determines how we extract the symbol for an object. This
 //     enables the 'symbolStyle' configuration option.
 func collectSymbols(ctx context.Context, views []View, matcherType SymbolMatcher, symbolizer symbolizer, query string) ([]protocol.SymbolInformation, error) {
-
 	// Extract symbols from all files.
 	var work []symbolFile
 	var roots []string
 	seen := make(map[span.URI]bool)
 	// TODO(adonovan): opt: parallelize this loop? How often is len > 1?
 	for _, v := range views {
-		snapshot, release := v.Snapshot(ctx)
+		snapshot, release, err := v.Snapshot()
+		if err != nil {
+			continue // view is shut down; continue with others
+		}
 		defer release()
 
 		// Use the root view URIs for determining (lexically)
@@ -315,7 +317,17 @@ func collectSymbols(ctx context.Context, views []View, matcherType SymbolMatcher
 		filters := v.Options().DirectoryFilters
 		filterer := NewFilterer(filters)
 		folder := filepath.ToSlash(v.Folder().Filename())
-		for uri, syms := range snapshot.Symbols(ctx) {
+
+		workspaceOnly := true
+		if v.Options().SymbolScope == AllSymbolScope {
+			workspaceOnly = false
+		}
+		symbols, err := snapshot.Symbols(ctx, workspaceOnly)
+		if err != nil {
+			return nil, err
+		}
+
+		for uri, syms := range symbols {
 			norm := filepath.ToSlash(uri.Filename())
 			nm := strings.TrimPrefix(norm, folder)
 			if filterer.Disallow(nm) {
@@ -325,17 +337,13 @@ func collectSymbols(ctx context.Context, views []View, matcherType SymbolMatcher
 			if seen[uri] {
 				continue
 			}
-			mds, err := snapshot.MetadataForFile(ctx, uri)
+			meta, err := NarrowestMetadataForFile(ctx, snapshot, uri)
 			if err != nil {
 				event.Error(ctx, fmt.Sprintf("missing metadata for %q", uri), err)
 				continue
 			}
-			if len(mds) == 0 {
-				// TODO: should use the bug reporting API
-				continue
-			}
 			seen[uri] = true
-			work = append(work, symbolFile{uri, mds[0], syms})
+			work = append(work, symbolFile{uri, meta, syms})
 		}
 	}
 
@@ -441,7 +449,7 @@ func convertFilterToRegexp(filter string) *regexp.Regexp {
 // symbolFile holds symbol information for a single file.
 type symbolFile struct {
 	uri  span.URI
-	md   Metadata
+	md   *Metadata
 	syms []Symbol
 }
 
@@ -465,18 +473,21 @@ func matchFile(store *symbolStore, symbolizer symbolizer, matcher matcherFunc, r
 		// All factors are multiplicative, meaning if more than one applies they are
 		// multiplied together.
 		const (
-			// nonWorkspaceFactor is applied to symbols outside of any active
-			// workspace. Developers are less likely to want to jump to code that they
+			// nonWorkspaceFactor is applied to symbols outside the workspace.
+			// Developers are less likely to want to jump to code that they
 			// are not actively working on.
 			nonWorkspaceFactor = 0.5
-			// nonWorkspaceUnexportedFactor is applied to unexported symbols outside of
-			// any active workspace. Since one wouldn't usually jump to unexported
+			// nonWorkspaceUnexportedFactor is applied to unexported symbols outside
+			// the workspace. Since one wouldn't usually jump to unexported
 			// symbols to understand a package API, they are particularly irrelevant.
 			nonWorkspaceUnexportedFactor = 0.5
 			// every field or method nesting level to access the field decreases
 			// the score by a factor of 1.0 - depth*depthFactor, up to a depth of
 			// 3.
-			depthFactor = 0.2
+			//
+			// Use a small constant here, as this exists mostly to break ties
+			// (e.g. given a type Foo and a field x.Foo, prefer Foo).
+			depthFactor = 0.01
 		)
 
 		startWord := true
@@ -494,6 +505,8 @@ func matchFile(store *symbolStore, symbolizer symbolizer, matcher matcherFunc, r
 			}
 		}
 
+		// TODO(rfindley): use metadata to determine if the file is in a workspace
+		// package, rather than this heuristic.
 		inWorkspace := false
 		for _, root := range roots {
 			if strings.HasPrefix(string(i.uri), root) {
@@ -526,7 +539,7 @@ func matchFile(store *symbolStore, symbolizer symbolizer, matcher matcherFunc, r
 			kind:      sym.Kind,
 			uri:       i.uri,
 			rng:       sym.Range,
-			container: i.md.PackagePath(),
+			container: string(i.md.PkgPath),
 		}
 		store.store(si)
 	}
